=== added file 'bzrlib/generate_ids.py'
--- bzrlib/generate_ids.py	1970-01-01 00:00:00 +0000
+++ bzrlib/generate_ids.py	2006-11-02 22:48:49 +0000
@@ -0,0 +1,119 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Common code for generating file or revision ids."""
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import time
+import unicodedata
+
+from bzrlib import (
+    config,
+    errors,
+    osutils,
+    )
+""")
+
+from bzrlib import (
+    lazy_regex,
+    )
+
+# the regex removes any weird characters; we don't escape them 
+# but rather just pull them out
+_file_id_chars_re = lazy_regex.lazy_compile(r'[^\w.]')
+_rev_id_chars_re = lazy_regex.lazy_compile(r'[^-\w.+@]')
+_gen_file_id_suffix = None
+_gen_file_id_serial = 0
+
+
+def _next_id_suffix():
+    """Create a new file id suffix that is reasonably unique.
+    
+    On the first call we combine the current time with 64 bits of randomness to
+    give a highly probably globally unique number. Then each call in the same
+    process adds 1 to a serial number we append to that unique value.
+    """
+    # XXX TODO: change bzrlib.add.smart_add to call workingtree.add() rather 
+    # than having to move the id randomness out of the inner loop like this.
+    # XXX TODO: for the global randomness this uses we should add the thread-id
+    # before the serial #.
+    # XXX TODO: jam 20061102 I think it would be good to reset every 100 or
+    #           1000 calls, or perhaps if time.time() increases by a certain
+    #           amount. time.time() shouldn't be terribly expensive to call,
+    #           and it means that long-lived processes wouldn't use the same
+    #           suffix forever.
+    global _gen_file_id_suffix, _gen_file_id_serial
+    if _gen_file_id_suffix is None:
+        _gen_file_id_suffix = "-%s-%s-" % (osutils.compact_date(time.time()),
+                                           osutils.rand_chars(16))
+    _gen_file_id_serial += 1
+    return _gen_file_id_suffix + str(_gen_file_id_serial)
+
+
+def gen_file_id(name):
+    """Return new file id for the basename 'name'.
+
+    The uniqueness is supplied from _next_id_suffix.
+    """
+    # The real randomness is in the _next_id_suffix, the
+    # rest of the identifier is just to be nice.
+    # So we:
+    # 1) Remove non-ascii word characters to keep the ids portable
+    # 2) squash to lowercase, so the file id doesn't have to
+    #    be escaped (case insensitive filesystems would bork for ids
+    #    that only differ in case without escaping).
+    # 3) truncate the filename to 20 chars. Long filenames also bork on some
+    #    filesystems
+    # 4) Removing starting '.' characters to prevent the file ids from
+    #    being considered hidden.
+    ascii_word_only = _file_id_chars_re.sub('', name.lower())
+    short_no_dots = ascii_word_only.lstrip('.')[:20]
+    return short_no_dots + _next_id_suffix()
+
+
+def gen_root_id():
+    """Return a new tree-root file id."""
+    return gen_file_id('tree_root')
+
+
+def gen_revision_id(username, timestamp=None):
+    """Return new revision-id.
+
+    :param username: This is the value returned by config.username(), which is
+        typically a real name, followed by an email address. If found, we will
+        use just the email address portion. Otherwise we flatten the real name,
+        and use that.
+    :return: A new revision id.
+    """
+    try:
+        user_or_email = config.extract_email_address(username)
+    except errors.NoEmailInUsername:
+        user_or_email = username
+
+    user_or_email = user_or_email.lower()
+    user_or_email = user_or_email.replace(' ', '_')
+    user_or_email = _rev_id_chars_re.sub('', user_or_email)
+
+    # This gives 36^16 ~= 2^82.7 ~= 83 bits of entropy
+    unique_chunk = osutils.rand_chars(16)
+
+    if timestamp is None:
+        timestamp = time.time()
+
+    return '-'.join((user_or_email,
+                    osutils.compact_date(timestamp),
+                    unique_chunk))

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-10-25 15:04:02 +0000
+++ bzrlib/inventory.py	2006-11-02 22:48:49 +0000
@@ -39,6 +39,7 @@
 import bzrlib
 from bzrlib import (
     errors,
+    generate_ids,
     osutils,
     symbol_versioning,
     )
@@ -1086,7 +1087,7 @@
 
         if len(parts) == 0:
             if file_id is None:
-                file_id = bzrlib.workingtree.gen_root_id()
+                file_id = generate_ids.gen_root_id()
             self.root = InventoryDirectory(file_id, '', None)
             self._byid = {self.root.file_id: self.root}
             return
@@ -1283,7 +1284,7 @@
     :param file_id: the file_id to use. if None, one will be created.
     """
     if file_id is None:
-        file_id = bzrlib.workingtree.gen_file_id(name)
+        file_id = generate_ids.gen_file_id(name)
 
     norm_name, can_access = osutils.normalized_filename(name)
     if norm_name != name:

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-10-12 01:50:46 +0000
+++ bzrlib/repository.py	2006-11-02 22:48:49 +0000
@@ -29,6 +29,7 @@
     check,
     delta,
     errors,
+    generate_ids,
     gpg,
     graph,
     knit,
@@ -2355,12 +2356,15 @@
             self.parents
             )
 
+    @staticmethod
+    def _sanitize_for_revision_id(text):
+        """This just removes whitespace, etc to make friendlier ids.
+        """
+
     def _gen_revision_id(self):
         """Return new revision-id."""
-        s = '%s-%s-' % (self._config.user_email(), 
-                        compact_date(self._timestamp))
-        s += hexlify(rand_bytes(8))
-        return s
+        return generate_ids.gen_revision_id(self._config.username(),
+                                            self._timestamp)
 
     def _generate_revision_if_needed(self):
         """Create a revision id if None was supplied.

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-11-02 08:37:46 +0000
+++ bzrlib/tests/__init__.py	2006-11-02 22:48:49 +0000
@@ -1690,6 +1690,7 @@
                    'bzrlib.tests.test_escaped_store',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_ftp_transport',
+                   'bzrlib.tests.test_generate_ids',
                    'bzrlib.tests.test_gpg',
                    'bzrlib.tests.test_graph',
                    'bzrlib.tests.test_hashcache',

=== added file 'bzrlib/tests/test_generate_ids.py'
--- bzrlib/tests/test_generate_ids.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_generate_ids.py	2006-11-02 22:48:49 +0000
@@ -0,0 +1,130 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzrlib/generate_ids.py"""
+
+import re
+
+from bzrlib import (
+    generate_ids,
+    tests,
+    )
+
+
+class TestFileIds(tests.TestCase):
+    """Test functions which generate file ids"""
+    
+    def test_gen_file_id(self):
+        gen_file_id = generate_ids.gen_file_id
+
+        # We try to use the filename if possible
+        self.assertStartsWith(gen_file_id('bar'), 'bar-')
+
+        # but we squash capitalization, and remove non word characters
+        self.assertStartsWith(gen_file_id('Mwoo oof\t m'), 'mwoooofm-')
+
+        # We also remove leading '.' characters to prevent hidden file-ids
+        self.assertStartsWith(gen_file_id('..gam.py'), 'gam.py-')
+        self.assertStartsWith(gen_file_id('..Mwoo oof\t m'), 'mwoooofm-')
+
+        # we remove unicode characters, and still don't end up with a 
+        # hidden file id
+        self.assertStartsWith(gen_file_id(u'\xe5\xb5.txt'), 'txt-')
+        
+        # Our current method of generating unique ids adds 33 characters
+        # plus an serial number (log10(N) characters)
+        # to the end of the filename. We now restrict the filename portion to
+        # be <= 20 characters, so the maximum length should now be approx < 60
+
+        # Test both case squashing and length restriction
+        fid = gen_file_id('A'*50 + '.txt')
+        self.assertStartsWith(fid, 'a'*20 + '-')
+        self.failUnless(len(fid) < 60)
+
+        # restricting length happens after the other actions, so
+        # we preserve as much as possible
+        fid = gen_file_id('\xe5\xb5..aBcd\tefGhijKLMnop\tqrstuvwxyz')
+        self.assertStartsWith(fid, 'abcdefghijklmnopqrst-')
+        self.failUnless(len(fid) < 60)
+
+    def test__next_id_suffix_sets_suffix(self):
+        generate_ids._gen_file_id_suffix = None
+        generate_ids._next_id_suffix()
+        self.assertNotEqual(None, generate_ids._gen_file_id_suffix)
+
+    def test__next_id_suffix_increments(self):
+        generate_ids._gen_file_id_suffix = "foo-"
+        generate_ids._gen_file_id_serial = 1
+        try:
+            self.assertEqual("foo-2", generate_ids._next_id_suffix())
+            self.assertEqual("foo-3", generate_ids._next_id_suffix())
+            self.assertEqual("foo-4", generate_ids._next_id_suffix())
+            self.assertEqual("foo-5", generate_ids._next_id_suffix())
+            self.assertEqual("foo-6", generate_ids._next_id_suffix())
+            self.assertEqual("foo-7", generate_ids._next_id_suffix())
+            self.assertEqual("foo-8", generate_ids._next_id_suffix())
+            self.assertEqual("foo-9", generate_ids._next_id_suffix())
+            self.assertEqual("foo-10", generate_ids._next_id_suffix())
+        finally:
+            # Reset so that all future ids generated in the test suite
+            # don't end in 'foo-XXX'
+            generate_ids._gen_file_id_suffix = None
+            generate_ids._gen_file_id_serial = 0
+
+    def test_gen_root_id(self):
+        # Mostly just make sure gen_root_id() exists
+        root_id = generate_ids.gen_root_id()
+        self.assertStartsWith(root_id, 'tree_root-')
+
+
+class TestGenRevisionId(tests.TestCase):
+    """Test generating revision ids"""
+
+    def assertMatchesRe(self, regex, text):
+        """Make sure text is matched by the regex given"""
+        if re.match(regex, text) is None:
+            self.fail('Pattern %s did not match text %s' % (regex, text))
+
+    def assertGenRevisionId(self, regex, username, timestamp=None):
+        """gen_revision_id should create a revision id matching the regex"""
+        revision_id = generate_ids.gen_revision_id(username, timestamp)
+        self.assertMatchesRe(regex, revision_id)
+
+    def test_timestamp(self):
+        """passing a timestamp should cause it to be used"""
+        self.assertGenRevisionId(r'user@host-\d{14}-[a-z0-9]{16}', 'user@host')
+        self.assertGenRevisionId('user@host-20061102205056-[a-z0-9]{16}',
+                                 'user@host', 1162500656.688)
+        self.assertGenRevisionId(r'user@host-20061102205024-[a-z0-9]{16}',
+                                 'user@host', 1162500624.000)
+
+    def test_gen_revision_id_email(self):
+        """gen_revision_id uses email address if present"""
+        regex = r'user\+joe_bar@foo-bar\.com-\d{14}-[a-z0-9]{16}'
+        self.assertGenRevisionId(regex,'user+joe_bar@foo-bar.com')
+        self.assertGenRevisionId(regex, '<user+joe_bar@foo-bar.com>')
+        self.assertGenRevisionId(regex, 'Joe Bar <user+joe_bar@foo-bar.com>')
+        self.assertGenRevisionId(regex, 'Joe Bar <user+Joe_Bar@Foo-Bar.com>')
+        self.assertGenRevisionId(regex, u'Joe B\xe5r <user+Joe_Bar@Foo-Bar.com>')
+
+    def test_gen_revision_id_user(self):
+        """If there is no email, fall back to the whole username"""
+        tail = r'-\d{14}-[a-z0-9]{16}'
+        self.assertGenRevisionId('joe_bar' + tail,'Joe Bar')
+        self.assertGenRevisionId('joebar' + tail, 'joebar')
+        self.assertGenRevisionId('joe_br' + tail, u'Joe B\xe5r')
+        self.assertGenRevisionId(r'joe_br_user\+joe_bar_foo-bar.com' + tail,
+                                 u'Joe B\xe5r <user+Joe_Bar_Foo-Bar.com>')

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-10-16 01:50:48 +0000
+++ bzrlib/tests/test_merge_core.py	2006-11-02 22:48:49 +0000
@@ -20,6 +20,7 @@
 
 import bzrlib
 from bzrlib import (
+    generate_ids,
     osutils,
     )
 from bzrlib.add import smart_add_tree
@@ -34,13 +35,13 @@
                             )
 from bzrlib.transform import TreeTransform
 from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
-from bzrlib.workingtree import WorkingTree, gen_root_id
+from bzrlib.workingtree import WorkingTree
 
 
 class MergeBuilder(object):
     def __init__(self, dir=None):
         self.dir = osutils.mkdtemp(prefix="merge-test", dir=dir)
-        self.tree_root = gen_root_id()
+        self.tree_root = generate_ids.gen_root_id()
         def wt(name):
            path = pathjoin(self.dir, name)
            os.mkdir(path)

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-10-29 01:44:29 +0000
+++ bzrlib/tests/test_transform.py	2006-11-02 22:48:49 +0000
@@ -20,6 +20,7 @@
 
 from bzrlib import (
     errors,
+    generate_ids,
     tests,
     urlutils,
     )
@@ -36,7 +37,6 @@
 from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths, 
                               resolve_conflicts, cook_conflicts, 
                               find_interesting, build_tree, get_backup_name)
-from bzrlib.workingtree import gen_root_id
 
 
 class TestTreeTransform(TestCaseInTempDir):
@@ -605,7 +605,7 @@
 
 class TestTransformMerge(TestCaseInTempDir):
     def test_text_merge(self):
-        root_id = gen_root_id()
+        root_id = generate_ids.gen_root_id()
         base = TransformGroup("base", root_id)
         base.tt.new_file('a', base.root, 'a\nb\nc\nd\be\n', 'a')
         base.tt.new_file('b', base.root, 'b1', 'b')
@@ -684,7 +684,7 @@
     def test_file_merge(self):
         if not has_symlinks():
             raise TestSkipped('Symlinks are not supported on this platform')
-        root_id = gen_root_id()
+        root_id = generate_ids.gen_root_id()
         base = TransformGroup("BASE", root_id)
         this = TransformGroup("THIS", root_id)
         other = TransformGroup("OTHER", root_id)
@@ -725,7 +725,7 @@
         self.assertIs(os.path.lexists(this.wt.abspath('h.OTHER')), True)
 
     def test_filename_merge(self):
-        root_id = gen_root_id()
+        root_id = generate_ids.gen_root_id()
         base = TransformGroup("BASE", root_id)
         this = TransformGroup("THIS", root_id)
         other = TransformGroup("OTHER", root_id)
@@ -758,7 +758,7 @@
         self.assertEqual(this.wt.id2path('f'), pathjoin('b/f1'))
 
     def test_filename_merge_conflicts(self):
-        root_id = gen_root_id()
+        root_id = generate_ids.gen_root_id()
         base = TransformGroup("BASE", root_id)
         this = TransformGroup("THIS", root_id)
         other = TransformGroup("OTHER", root_id)

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-10-17 08:14:59 +0000
+++ bzrlib/tests/test_workingtree.py	2006-11-02 22:48:49 +0000
@@ -27,6 +27,7 @@
 from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
+from bzrlib.symbol_versioning import zero_thirteen
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
@@ -258,56 +259,15 @@
 class TestNonFormatSpecificCode(TestCaseWithTransport):
     """This class contains tests of workingtree that are not format specific."""
 
-    
     def test_gen_file_id(self):
-        gen_file_id = bzrlib.workingtree.gen_file_id
-
-        # We try to use the filename if possible
-        self.assertStartsWith(gen_file_id('bar'), 'bar-')
-
-        # but we squash capitalization, and remove non word characters
-        self.assertStartsWith(gen_file_id('Mwoo oof\t m'), 'mwoooofm-')
-
-        # We also remove leading '.' characters to prevent hidden file-ids
-        self.assertStartsWith(gen_file_id('..gam.py'), 'gam.py-')
-        self.assertStartsWith(gen_file_id('..Mwoo oof\t m'), 'mwoooofm-')
-
-        # we remove unicode characters, and still don't end up with a 
-        # hidden file id
-        self.assertStartsWith(gen_file_id(u'\xe5\xb5.txt'), 'txt-')
+        file_id = self.applyDeprecated(zero_thirteen, workingtree.gen_file_id,
+                                      'filename')
+        self.assertStartsWith(file_id, 'filename-')
+
+    def test_gen_root_id(self):
+        file_id = self.applyDeprecated(zero_thirteen, workingtree.gen_root_id)
+        self.assertStartsWith(file_id, 'tree_root-')
         
-        # Our current method of generating unique ids adds 33 characters
-        # plus an serial number (log10(N) characters)
-        # to the end of the filename. We now restrict the filename portion to
-        # be <= 20 characters, so the maximum length should now be approx < 60
-
-        # Test both case squashing and length restriction
-        fid = gen_file_id('A'*50 + '.txt')
-        self.assertStartsWith(fid, 'a'*20 + '-')
-        self.failUnless(len(fid) < 60)
-
-        # restricting length happens after the other actions, so
-        # we preserve as much as possible
-        fid = gen_file_id('\xe5\xb5..aBcd\tefGhijKLMnop\tqrstuvwxyz')
-        self.assertStartsWith(fid, 'abcdefghijklmnopqrst-')
-        self.failUnless(len(fid) < 60)
-
-    def test_next_id_suffix(self):
-        bzrlib.workingtree._gen_id_suffix = None
-        bzrlib.workingtree._next_id_suffix()
-        self.assertNotEqual(None, bzrlib.workingtree._gen_id_suffix)
-        bzrlib.workingtree._gen_id_suffix = "foo-"
-        bzrlib.workingtree._gen_id_serial = 1
-        self.assertEqual("foo-2", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-3", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-4", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-5", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-6", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-7", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-8", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-9", bzrlib.workingtree._next_id_suffix())
-        self.assertEqual("foo-10", bzrlib.workingtree._next_id_suffix())
-
     def test__translate_ignore_rule(self):
         tree = self.make_branch_and_tree('.')
         # translation should return the regex, the number of groups in it,

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-10-19 00:05:23 +0000
+++ bzrlib/workingtree.py	2006-11-02 22:48:49 +0000
@@ -55,6 +55,7 @@
     bzrdir,
     conflicts as _mod_conflicts,
     errors,
+    generate_ids,
     ignores,
     merge,
     osutils,
@@ -111,61 +112,30 @@
         DEPRECATED_PARAMETER,
         zero_eight,
         zero_eleven,
+        zero_thirteen,
         )
 
 
 MERGE_MODIFIED_HEADER_1 = "BZR merge-modified list format 1"
 CONFLICT_HEADER_1 = "BZR conflict list format 1"
 
-# the regex removes any weird characters; we don't escape them 
-# but rather just pull them out
-_gen_file_id_re = re.compile(r'[^\w.]')
-_gen_id_suffix = None
-_gen_id_serial = 0
-
-
-def _next_id_suffix():
-    """Create a new file id suffix that is reasonably unique.
-    
-    On the first call we combine the current time with 64 bits of randomness
-    to give a highly probably globally unique number. Then each call in the same
-    process adds 1 to a serial number we append to that unique value.
-    """
-    # XXX TODO: change bzrlib.add.smart_add to call workingtree.add() rather 
-    # than having to move the id randomness out of the inner loop like this.
-    # XXX TODO: for the global randomness this uses we should add the thread-id
-    # before the serial #.
-    global _gen_id_suffix, _gen_id_serial
-    if _gen_id_suffix is None:
-        _gen_id_suffix = "-%s-%s-" % (compact_date(time()), rand_chars(16))
-    _gen_id_serial += 1
-    return _gen_id_suffix + str(_gen_id_serial)
-
-
+
+@deprecated_function(zero_thirteen)
 def gen_file_id(name):
     """Return new file id for the basename 'name'.
 
-    The uniqueness is supplied from _next_id_suffix.
+    Use bzrlib.generate_ids.gen_file_id() instead
     """
-    # The real randomness is in the _next_id_suffix, the
-    # rest of the identifier is just to be nice.
-    # So we:
-    # 1) Remove non-ascii word characters to keep the ids portable
-    # 2) squash to lowercase, so the file id doesn't have to
-    #    be escaped (case insensitive filesystems would bork for ids
-    #    that only differred in case without escaping).
-    # 3) truncate the filename to 20 chars. Long filenames also bork on some
-    #    filesystems
-    # 4) Removing starting '.' characters to prevent the file ids from
-    #    being considered hidden.
-    ascii_word_only = _gen_file_id_re.sub('', name.lower())
-    short_no_dots = ascii_word_only.lstrip('.')[:20]
-    return short_no_dots + _next_id_suffix()
-
-
+    return generate_ids.gen_file_id(name)
+
+
+@deprecated_function(zero_thirteen)
 def gen_root_id():
-    """Return a new tree-root file id."""
-    return gen_file_id('tree_root')
+    """Return a new tree-root file id.
+
+    This has been deprecated in favor of bzrlib.generate_ids.gen_root_id()
+    """
+    return generate_ids.gen_root_id()
 
 
 class TreeEntry(object):
@@ -850,7 +820,7 @@
     def mkdir(self, path, file_id=None):
         """See MutableTree.mkdir()."""
         if file_id is None:
-            file_id = gen_file_id(os.path.basename(path))
+            file_id = generate_ids.gen_file_id(os.path.basename(path))
         os.mkdir(self.abspath(path))
         self.add(path, file_id, 'directory')
         return file_id

